import signal
import time

from jupyter_ascending.logger import J_LOGGER
from jupyter_ascending.server_multiplexer import start_server_in_thread
from jupyter_ascending.syncer import start_notebook_server_in_thread
from jupyter_ascending.utils import get_name_from_python
from jupyter_ascending.widget import SyncMagic


def load_ipython_extension(ipython):
    # Add %start_notebook_syncing
    ipython.register_magics(SyncMagic)

    # Start the server if it's the right name.
    notebook_name = get_name_from_python()

    if ".synced.ipynb" not in notebook_name:
        J_LOGGER.info("IPYTHON: Note loading {notebook} because name does not match", notebook=notebook_name)
        return

    J_LOGGER.info("IPYTHON LOAD: " + time.ctime() + ": " + notebook_name)
    start_notebook_server_in_thread(notebook_name)


def load_jupyter_server_extension(ipython):
    J_LOGGER.info("SERVER LOAD: " + time.ctime())

    server = start_server_in_thread()

    # HACK:
    # A bit of a hack to make sure the server gets shutdown when we're done here.
    #   Had some problems with hanging servers
    #
    # I think this doesn't quite work if we don't confirm that we want the server shutdown.
    #   Oh well for now...
    ORIGINAL = None

    def shutdown_from_signal(*args, **kwargs):
        if ORIGINAL:
            ORIGINAL(*args, **kwargs)

        J_LOGGER.info("SERVER: Shutting down server")
        server.shutdown()

    ORIGINAL = signal.signal(signal.SIGINT, shutdown_from_signal)
